# Monochrome Films Light Fogging Correction Support tool for ImageJ
# Ver. 0.07 version
# write out R-G dif value
# make some processes sub routines
# add input Power for Mixing Tuning
# read parameters from parameter.txt
# power for left & right
# (c)2021-22 Ohnishi, Yasuo
from ij import IJ, ImagePlus, CompositeImage
from ij import WindowManager as WM
from ij.io import DirectoryChooser, OpenDialog
from ij.gui import GenericDialog
from ij.plugin import ChannelSplitter, Duplicator
from ij.plugin import RGBStackMerge, RGBStackConverter
from ij.process import ImageConverter, ImageProcessor
from loci.plugins import BF
import math
import os
import sys
from org.python.core import codecs
codecs.setDefaultEncoding('utf-8')
import datetime
from java.awt.event import ItemListener
from java.lang import Integer

def convValue(Value, Mode):
#   Converting Luminance Value each in 8, 16, 32bit mode
    if Mode == "16-bit":
        Value = Value * 256
    elif Mode == "32-bit":
        Value = Value * 256 * 256
    else:
        Value = Value
    return Value

####### calculating mean L Value in Sampling Region  ############
def calcMean(imp0, xStart, xEnd, nYDivide, yStart, yEnd, folder):
#   imp0: image / xStart & xEnd: Starting & Ending Point (x) of ROI
#   nYDivide:  number of dividing zones in Y
    nYDivide = int(nYDivide) # number of dividing zones in Y
    width = imp0.getWidth() # get image width
    height = imp0.getHeight() # get image height
    pixelNum = width*height # get total pixel number
    print ("W x H (imp0)", str(width), str(height))
    if yStart == 0 and yEnd == 0:
        height2 = height
    else:
        height2 = yEnd - yStart
    zoneBreadth = height2 // nYDivide # divided Y zone height
    tmpsumDiff = [0] * nYDivide # sum of R-G differance in a zone
    n_tmpsumDiff = [0] * nYDivide # sum of pixel number in a zone
    mean2 = [0.0] * nYDivide # mean R-G differance in a zone

#w    writing_file1 = open(folder + 'mean2.txt','w')
#w    writing_file1.write("Mean xStart: " + str(xStart) + " xEnd: " + str(xEnd) + "\n")
#w    writing_file1.write("original value" + "\n")

    for x in range(xStart, xEnd): # processing in each X row
#       ----   
        for yz in range(nYDivide): # processing in each Y zone
            for yj in range(yStart + yz * zoneBreadth, yStart + (yz + 1) * zoneBreadth): 
                Px0 = imp0.getPixel(x, yj) # get R pixel Value
                tmpsumDiff[yz] = tmpsumDiff[yz] + Px0[0] # summate Value in each Y Zone
#w                if (yz == 11) or (yz == 12):
#w                    writing_file1.write(str(yz) + "," + str(x) + "," + str(yj) + "," + str(Px0[0]) + "\n")
                n_tmpsumDiff[yz] = n_tmpsumDiff[yz] + 1 # count pixel number in each Y Zone
#       ----   
#    for inspecting contents of mean2
#w    writing_file1.write("mean value" + "\n")
    for yz in range(nYDivide): # calculate mean R-G difference in each Y Zone
        mean2[yz] = float(tmpsumDiff[yz]) / float(n_tmpsumDiff[yz])
#w        writing_file1.write(str(yz) + "," + str(mean2[yz]/256) + "\n")
#       ----   
#w    writing_file1.close()
    return mean2 # Return value: mean[n of blocks] R-G difference in Y Zones
#---------------
#### Getting L value in each columns in ROI
def get_RG_v_diff(imp0, xStart, xEnd, yStart, yEnd, height2, folder, nDivide):
    tmpMxDiffRG = [[0.0 for yj in range(height2)] for xi in range(xEnd - xStart)]
    timenow = datetime.datetime.now()
    print("Start Read tmpMxDiffRG:", timenow)
    for x in range(xStart, xEnd): # process in each x col
        for y in range(yStart, yStart + height2):
#                   get imp0 image pixel value
            Px0 = imp0.getPixel(x, y) 
            tmpMxDiffRG[x - xStart][y - yStart] = Px0[0]
                 # save pixel value to tmpMxDiffRG
    """
    # for inspecting contents of tmpMxDiffRG
    tmpMxDiffRG_writeout(tmpMxDiffRG, folder, nDivide)
    """
    return tmpMxDiffRG
     # tmpMxDiffRG[width pixes] x [n of blocks]: Pixel Luminance value in ROI 
#------------------------------
def tmpMxDiffRG_writeout(tmpMxDiffRG, folder, nDivide):
    writing_file2 = open(folder + 'tmpMxDiffRG.txt','w')
    xlen = len(tmpMxDiffRG)
    writing_file2.write("xlen: " + str(xlen) + "\n")
    ylen = [len(v) for v in tmpMxDiffRG]
    # print ("ylen:", ylen)
    writing_file2.write("ylen: " + str(ylen) + "\n")
    zoneBreadth = ylen[0] // nDivide
#    """
    for x in range(xlen):
        if x == int(xlen / 2):
            for yi in range(nDivide):
                sum = 0
                n = 0
                for y in range(yi * zoneBreadth, (yi + 1) * zoneBreadth):
                    sum = sum + tmpMxDiffRG[x][y]
                    n = n + 1
                sum = sum / n
                writing_file2.write(str(yi) + ',' + str(sum / 256) + "\n")
#    """
    writing_file2.close()

#---------------------------------
#### Calculating correcting value
def calculate_correct_value(tmpMxDiffRG, nYDivide, xStart, xEnd, yStart, yEnd, height2, mean2, columsWidth):
    """
      height2: height of ROI (unit: pixels)
      mean2[n of blocks]: mean Luminance value in blocks
      tmpMxDiffRG[width pixes] x [n of blocks]: Luminance value in ROI 
      nYDivide: number of divided block for calculation
      xStart, xEnd, yStart, yEnd: coordinate of ROI
    """
    maxcorrectValue = 0
    maxCorV = [0.0 for yz in range(nYDivide)]
    correctValueMx = [[0.0 for yz in range(nYDivide)] for xi in range(xEnd - xStart)]
    zoneBreadth = height2 // nYDivide # height of a Y zone
    print ("zoneBreadth: ", zoneBreadth)
    timenow = datetime.datetime.now()
    print("Start Calculate correctValueMx:", timenow)
#### Calculating correcting value
    for xi in range(xEnd - xStart): # process in each x colums in ROI
        for yz in range(nYDivide):
            sumtmpDiffRG = 0.0
            nr = 0.0
            if (xi == 0) and (yz == int(nYDivide /2)):
                timenow = datetime.datetime.now()
                print("Calcurate ", xi, "th correctValueMx:", timenow)
            for xi2 in range(xi - columsWidth, xi+ columsWidth + 1): # calculating back and forth 5 rows
                for yj in range(yStart + yz * zoneBreadth, yStart + (yz + 1) * zoneBreadth): 
                    if (xi2 > 0) and (yj > 0) and (xi2 < (xEnd - xStart)) and (yj < yStart + height2): 
                       # summate L value in each Y zone
                        nr = nr + 1.0
                        sumtmpDiffRG = sumtmpDiffRG + tmpMxDiffRG[xi2][yj - yStart] #
            if nr > 0.0:
                correctValueMx[xi][yz] = (sumtmpDiffRG / nr) - mean2[yz]
#                correctValueMx[xi][yz] = abs(sumtmpDiffRG / nr) - abs(mean2[yz])  # 2022.5.27 modified not good
#       define correcting value
#           divide sum of value with pixel number and subtract normal value mean
            else: 
                correctValueMx[xi][yz] = 0.0
            if correctValueMx[xi][yz] < 0.0: # if correction value < 0.0 in a Y zone then no correction
                correctValueMx[xi][yz] = 0.0 
            if (xi % 100 == 0) and (yz == 5):
                 print ("xi: ", xi, "yz: ", yz, "correctValueMx: ", correctValueMx[xi][yz])
            maxcorrectValue = max(maxcorrectValue, correctValueMx[xi][yz]) # calculate max correction value
            maxCorV[yz] = max(maxCorV[yz], correctValueMx[xi][yz]) # calculate max correction value
    print ("maxcorrectValue", maxcorrectValue)
    return maxcorrectValue, correctValueMx, maxCorV, zoneBreadth
    """
        max correctValue:
        correctValueMx[width pixels of ROI][n of blocks]:
        zoneLUT[height pixels of ROI]:
        maxCorV[n of blocks]:
    """
#-----------
# image correction processing log writing out
def image_correct_log_writeout(x, y, height2, yStart, Pcc, HPcc, MPcc, LPcc):
    if (x % 100 == 0) and (y == int(height2)/5 + yStart):
        HPcc = Pcc
    elif (x % 100 == 0) and (y == int(height2)/2 + yStart):
        MPcc = Pcc
    elif (x % 100 == 0) and (y == 4 * int(height2)/5 + yStart):
        LPcc = Pcc
        print("x(",x,")  G Mixing Ratio L:", LPcc, "M: ",MPcc, "H: ",HPcc)
    return  HPcc, MPcc, LPcc

######### Image correcting routine ################
def image_correct(imp0, imp2, xStart, xEnd, Mode, nYDivide, mean2, yStart, yEnd, folder, mixFactor, columsWidth, smoothingBlocks, smoothingX, powerSmX, powerSmXR): 
# imp0: original Image imp2: R image to be corrected mean2: mean L Value
# xStart/xEnd: x start/end value of ROI, Mode: bit depth of image: mixFactor: scaling factor of correction
# nYDivide: dividing number of ROI in Y direction
# mean2: mean L value in each divided ROIs 
    nYDivide = int(nYDivide) # number of Y zone
    width = imp0.getWidth() # get image width
    if xEnd > width:
        xEnd = width
    height = imp0.getHeight() # get image height
    if yStart == 0 and yEnd == 0:
        height2 = height
    else:
        height2 = yEnd - yStart
    imp2Width = imp2.getWidth()
    imp2Height  = imp2.getHeight()
#w    IJ.showMessage("imp2 width and height: " + str(imp2Width) + " " + str(imp2Height))
    IJ.showMessage("Start calculation of correcting values")
    correctValue = []
#### Getting R-G value difference in each pixel
    tmpMxDiffRG = get_RG_v_diff(imp0, xStart, xEnd, yStart, yEnd, height2, folder, nYDivide)
#### Calculating correcting value
    maxcorrectValue, correctValueMx, maxCorV, zoneBreadth = calculate_correct_value(tmpMxDiffRG, nYDivide, xStart, xEnd, yStart, yEnd, height2, mean2, columsWidth)

### value correction process
    timenow = datetime.datetime.now()
    print("Start Correction:", timenow)
    HPcc = 0.0
    MPcc = 0.0
    LPcc = 0.0
#w    IJ.showMessage(str(xStart) + " " + str(xEnd) + " " + str(yStart) + " " + str(height2))
    ymedian = zoneBreadth // 2
    xmedian = (xEnd - xStart) / 2
#w    IJ.showMessage("ymedian: " + str(ymedian))
#w    IJ.showMessage("Smoothing y :" + str(smoothingBlocks))
    IJ.showMessage("Start Value Correction")
    for x in range(xStart, xEnd): # in each x row in ROI
        xSmoothingFactor = 1.0 - float(abs((x - xStart) - xmedian)) / float(xmedian)
        for yz in range(nYDivide):
            if correctValueMx[x - xStart][yz] < 0.0: # if correction value < 0 then no correction
                correctValueMx[x - xStart][yz] = 0.0
            Pcc0 = abs(correctValueMx[x - xStart][yz]) / maxcorrectValue
            if yz < nYDivide - 1:
               PccNext = abs(correctValueMx[x - xStart][yz + 1]) / maxcorrectValue
            else:
               PccNext = 0.0
            if yz > 0:
               PccPre = abs(correctValueMx[x - xStart][yz - 1]) / maxcorrectValue
            else:
               PccPre = 0.0

            for y in range(yStart + yz * zoneBreadth, yStart + (yz + 1) * zoneBreadth): # start value correction
                Px0= imp0.getPixel(x, y)

                """
                  Pcc0: Correcting Ratio of the Pixel to Max Correction Value
                """
    #             calculate correction value ratio
                y_inner = y - (yStart + yz * zoneBreadth)
                 # calcurating smoothing factor between calcuration blocks
                if y_inner < ymedian:  # smoothing factor with pre block
                    pcctunefac = float(y_inner) / float(ymedian)
                    Pcc1 = (Pcc0 + Pcc0 * pcctunefac + PccPre * (1-pcctunefac)) / 2.0
                else:                  # smoothing factor with next block
                    pcctunefac = float(y_inner - ymedian) / float(ymedian)
                    Pcc1 = (Pcc0 + PccNext * pcctunefac + Pcc0 * (1-pcctunefac)) / 2.0
                if smoothingBlocks != False: # execute smoothing between blocks
                    Pcc = Pcc1 * mixFactor #  adjusting correctioning amount with mixFactor
                else:                       # no smoothing
                    Pcc = Pcc0 * mixFactor #  adjusting correctioning amount with mixFactor
                if smoothingX ==True:
                    if x <= xStart + xmedian:
                        Pcc = Pcc * (xSmoothingFactor ** powerSmX)
                    else:
                        Pcc = Pcc * (xSmoothingFactor ** powerSmXR)
                """
                  Pcc: Actual G ch. Mixing rate for the Pixel
                """
    #             define corrected pixel luminance value 
                Pxc = Px0[0] - (maxcorrectValue * Pcc) # Correct L value with ratio Pcc
                """
                  Pxc: Actual Corrected R ch. Value of the Pixel
                """
                # write out processing log 
                HPcc, MPcc, LPcc = image_correct_log_writeout(x, y, height2, yStart, Pcc, HPcc, MPcc, LPcc)
                # set Pxc to Luminance. / Pxc: corrected value for Luminance.
                if Pxc <= 0:
                    Pxc = 0
                elif Pxc >= (256 * convValue(1, Mode) - 1):
                    Pxc = (256 * convValue(1, Mode) - 1)
                # print x, y, int(Pxc)
                if x < imp2Width and y <imp2Height:
                    imp2.getProcessor().set(x, y, int(Pxc))
    return imp2
#

#----------------------------------------------
def manipulation(imp0, imp2, Mode, Show, RegionSE, folder, filename, nYDivide, mixFactor, columsWidth, smoothingBlocks, smoothingX, powerSmX, powerSmXR, file_number):
# mixFactor: Scaling factor of correction amount
# powerSmX: Power for X direction smoothing factor 
# nYDivide: n of calcurating blocks (from parameter input dialog)
# columsWidth: Colums width for caliculation of correction value
# smoothingBlocks: execute or not smoothing for Y direction
# smoothingX: execute or not smoothing for X direction
# file_number: correction file sequence number 
    """
      RegionSE[x][y] coordinates of ROI and Sampling Region
                 [y]: 0: ROI xStart, 1: ROI xEnd, 2: ROI yStart, 3: ROI yEnd
                      4: sampling Region xStart 5: sampling Region xEnd
              [x]     0: 1st ROI(& Sampling Region), 1: 2nd ROI, 2: 3rd ROI
    """
    #imp0 = imps[0] #imp0: Original Image 
    if Mode == "Auto":
        bitdepth = imp0.getBitDepth()
        if bitdepth == 8:
            Mode = "8-bit"
        elif bitdepth ==16:
            Mode = "16-bit"
        elif bitdepth ==32:
            Mode = "32-bit"
        else:
            Mode = "16-bit"
    if Show == "Yes":
        imp0.show()
    if Show == "Yes":
        imp0.show()
    width = imp0.getWidth()
    iwidth = int(width * 0.015) # define width (15%) of sampling region for calculating mean R-G difference
    imp0Width = imp0.getWidth()
    imp0Height  = imp0.getHeight()
#w    IJ.showMessage("imp0 width and height(0): " + str(imp0Width) + " " + str(imp0Height))
    imp2Width = imp2.getWidth()
    imp2Height  = imp2.getHeight()
#w    IJ.showMessage("imp2 width and height(0): " + str(imp2Width) + " " + str(imp2Height))

    i = 0
    while i < 3:
      #  IJ.showMessage(str(i) + "th routine " + str(RegionSE[i][1]) + " " + str(RegionSE[i][0]))
        if RegionSE[i][1] - RegionSE[i][0] >= 1:
      #
            if RegionSE[i][5] - RegionSE[i][4] > 0:   #RegionSE[i][4]: SmplXEnd - RegionSE[i][5]: SmplXStart
                mean2 = calcMean(imp0, RegionSE[i][4], RegionSE[i][5], nYDivide, RegionSE[i][2], RegionSE[i][3], folder)
                                                           # RegionSE[i][2], RegionSE[i][3]: yStart, yEnd
            else:
                mean2 = calcMean(imp0, RegionSE[i][0]-iwidth, RegionSE[i][0]-1, nYDivide, RegionSE[i][2], RegionSE[i][3], folder)
          #  IJ.showMessage(str(i) + "th correcting routine")
            #image_correct(impR, impR2, impG,  RegionSE[i][0], RegionSE[i][1], Mode, GmixRatio, G_gamma2[i], nYDivide, mean2, RegionSE[i][2], RegionSE[i][3], folder, mixFactor)
            image_correct(imp0, imp2, RegionSE[i][0], RegionSE[i][1], Mode, nYDivide, mean2, RegionSE[i][2], RegionSE[i][3], folder, mixFactor, columsWidth, smoothingBlocks, smoothingX, powerSmX, powerSmXR)
            i = i + 1
        else:
            break


    imp2.show()
### save corrected image file
    IJ.saveAsTiff(imp2, folder + filename +"_correct(" + str(file_number) + ").tif")

    if Show == "Yes":
        imp0.hide()
        imp2.hide()
    imp0.close()
    imp2.close()

# ---ROI dialog-------
class RoiPreviewer(ItemListener):
    def __init__(self, imp, ckbox1, nfield_xStart, nfield_xEnd, nfield_yStart, nfield_yEnd):
        """
             imp: an ImagePlus
             preview_checkbox: a java.awt.Checkbox controlling whether to
                                                 dynamically update the ImagePlus as the
                                                 scrollbar is updated, or not.
        """
        self.imp = imp
        self.original_ip = imp.getProcessor().duplicate() # store a copy
        self.nfield_xStart = nfield_xStart
        self.nfield_xEnd = nfield_xEnd
        self.nfield_yStart = nfield_yStart
        self.nfield_yEnd = nfield_yEnd
        self.prv_ckBox1 = ckbox1
    

    def itemStateChanged(self, event):
        """ event: an ItemEvent with data on what happened to the checkbox. """
        if event.getStateChange() == event.SELECTED:
            self.changeRoi()
    
    def reset(self):
        """ Restore the original ImageProcessor """
        self.imp.setProcessor(self.original_ip)
    
    def changeRoi(self):
        """ Execute the in-place scaling of the ImagePlus. """
        rx = Integer.parseInt(self.nfield_xStart.getText())
        ry = Integer.parseInt(self.nfield_yStart.getText())
        rxe = Integer.parseInt(self.nfield_xEnd.getText())
        rye = Integer.parseInt(self.nfield_yEnd.getText())
        rw = rxe - rx
        rh = rye - ry
        rx = int(rx)
        ry = int(ry)
        rw = int(rw)
        rh = int(rh)
        if rx + rw > self.imp.width:
           rw = self.imp.width - rx
        if ry + rh > self.imp.height:
           rh = self.imp.height - ry
        print (rx, ry, rw, rh)
        self.imp.setRoi(rx, ry, rw, rh)

def setRoiUI(imp, i, Title, Message, yStart, yEnd, Mode, ROI_coord, Smpl_coord):
#    imp = WM.getCurrentImage()
    impw = imp.width
    imph = imp.height
    roi = imp.getRoi()
    # IJ.showMessage("Smpl:" + str(Smpl_coord["xEnd"] + Smpl_coord["xStart"]))
    if roi: # ROI is existing
        if (Title != "Select Sampling Region") and ((ROI_coord["xEnd"] + ROI_coord["xStart"]) == 0):
            # target ROI and no existing parameters
            rOrgx = roi.getBounds().x
            rOrgw = roi.getBounds().width
        elif (Title != "Select Sampling Region") and ((ROI_coord["xEnd"] + ROI_coord["xStart"]) > 0):
            # target ROI and parameters existing 
            rOrgx = ROI_coord["xStart"]
            rOrgw = ROI_coord["xEnd"] - ROI_coord["xStart"]
        elif (Smpl_coord["xEnd"] + Smpl_coord["xStart"]) > 0:
            # sampling ROI and parameters existing
            if roi.getBounds().x == ROI_coord["xStart"]: 
                    # if you don't change ROI > keep sampling ROI
                rOrgx = Smpl_coord["xStart"]
                rOrgw = Smpl_coord["xEnd"] - Smpl_coord["xStart"]
            else:   # if you change ROI > newly select sampling ROI
                rOrgx = max (roi.getBounds().x - roi.getBounds().width, 0)
                if rOrgx == 0:
                    rOrgw = roi.getBounds().x - 0
                else:
                    rOrgw = roi.getBounds().width
        else:
            # sampling ROI and no existing parameters
            rOrgx = max (roi.getBounds().x - roi.getBounds().width, 0)
            if rOrgx == 0:
                rOrgw = roi.getBounds().x - 0
            else:
                rOrgw = roi.getBounds().width
        rOrgy = roi.getBounds().y
        rOrgh = roi.getBounds().height
    else: # No ROI
        if Title != "Select Sampling Region":
            rOrgx = ROI_coord["xStart"]
            rOrgw = ROI_coord["xEnd"] - ROI_coord["xStart"]
        else:
            rOrgx = Smpl_coord["xStart"]
            rOrgw = Smpl_coord["xEnd"] - Smpl_coord["xStart"]
        rOrgy = yStart
        rOrgh = yEnd - yStart
    
    if not imp:
        print "Open an image first!"
        Mode = "Stop"
        return
    gd = GenericDialog(Title)
    gd.addMessage(Message + "  Image Pixel Size: " + str(impw) + " x " + str(imph))
    if Title == "Select Sampling Region":
        gd.addMessage("  Target ROI x bounds: " +  str(roi.getBounds().x) + " - " + str(roi.getBounds().x + roi.getBounds().width))
    gd.addNumericField("Start Point (x)", rOrgx, 0)
    gd.addNumericField("End Point (x)", rOrgx + rOrgw, 0)
    gd.addMessage("Y Region to be corrected")
    gd.addNumericField("Start Point (y)", yStart, 0)
    gd.addNumericField("End Point (y)", yEnd, 0)
    gd.addCheckbox("Preview ROI", False)
    gd.addMessage("If you would like to renew preview, \n clear checkbox once and check it again.")
    if Title != "Select Sampling Region":
#        gd.addNumericField("Gamma Correction Value for G (darken > 1.0 / Lighten <1.0)", 1.00, 2)
        Choice1 = ["Yes","No"]
        if Smpl_coord["xEnd"] + Smpl_coord["xStart"] == 0:
            gd.addChoice("Define Sampling Normal Region ?", Choice1, Choice1[1])
        else:
            gd.addChoice("Define Sampling Normal Region ?", Choice1, Choice1[0])
        Choice2 = ["Yes","No"]
        gd.addChoice("Define Next ROI?", Choice2, Choice2[1])

    # The UI elements for the above two inputs
    nfield_xStart = gd.getNumericFields().get(0) # 1st start
    nfield_xEnd = gd.getNumericFields().get(1) # 1st end
    nfield_yStart = gd.getNumericFields().get(2) # 1st start
    nfield_yEnd = gd.getNumericFields().get(3) # 1st end

    checkbox1 = gd.getCheckboxes().get(0)


    previewer = RoiPreviewer(imp, checkbox1, nfield_xStart, nfield_xEnd, nfield_yStart, nfield_yEnd)
    checkbox1.addItemListener(previewer)

    gd.showDialog()

    if gd.wasCanceled():
        previewer.reset()
        Mode   = "Stop"
        IJ.log("User Canceled")
        checkbox1.removeItemListener(previewer)
    else:
        previewer.changeRoi()
        checkbox1.removeItemListener(previewer)
    xStart =  gd.getNextNumber()
    xEnd =  gd.getNextNumber()
#    IJ.showMessage("xStart xEnd: " + str(xStart) + str(xEnd))
    if Title != "Select Sampling Region":
        yStart =  gd.getNextNumber()
        yEnd =  gd.getNextNumber()
#        IJ.showMessage("yStart yEnd: " + str(yStart) + str(yEnd))
        smplYes = gd.getNextChoice()
        Next = gd.getNextChoice()
    if Mode == "Stop":
        impclose = IJ.getImage()
        impclose.hide()
        impclose.close()
    if Title != "Select Sampling Region":
        return Mode, int(xStart), int(xEnd), int(yStart), int(yEnd), smplYes, Next
    else:
        return Mode, int(xStart), int(xEnd)

#--parameter dialog 
def getParameters_0():
    listRSE = [[0 for i in range(2)] for j in range(3)]
    gd = GenericDialog("Light Fog Correction Tool for monochrome film: (c)2022 Yasuo Ohnishi")
    gd.addNumericField("Factor for mixing adjustment amount", 1.00, 2)
    gd.addMessage("Vertical Dividing Zone")
    gd.addNumericField("Number of Y Dividing Zone", 25, 0)
    gd.addNumericField("Columns Width for calcurating correcting value", 10, 0)
    gd.addCheckbox("Smoothing between correction blocks", True)
    gd.addCheckbox("Smoothing X direction", True)
    gd.addNumericField("Power for Smoothing X (Left)", 0.20, 2)
    gd.addNumericField("Power for Smoothing X (Right)", 0.20, 2)
    Show = ["Yes","No"]
    gd.addChoice("Display working windows", Show, Show[0])
    gd.showDialog()
#
    parm_Mode   = "Auto"
    mixFactor  = gd.getNextNumber()
    nYDivide = gd.getNextNumber()
    columsWidth = gd.getNextNumber()
    smoothingBlocks = gd.getNextBoolean() #get value of checkbox
    smoothingX = gd.getNextBoolean() #get value of checkbox
    powerSmX = gd.getNextNumber()
    powerSmXR = gd.getNextNumber()
    parm_Show   = gd.getNextChoice()
    """
    if GmixRatio > 1.0:  # Restrict Maximum G ch. Mixing Ratio
        GmixRatio = 1.0
    """
#
    if gd.wasCanceled():  
        parm_Mode   = "Stop"
        impclose = IJ.getImage()
        impclose.hide()
        impclose.close()
        IJ.log("Canceled")
#
    return str(parm_Mode), str(parm_Show), nYDivide, mixFactor, int(columsWidth), smoothingBlocks, smoothingX, powerSmX, powerSmXR

#--------------------------------------
def read_parameter_txt(path):
    is_file = os.path.isfile(path + '_parameter.txt')
    file_number = 0
    ROI_coord_xStart = 0
    ROI_coord_xEnd = 0
    ROI_coord_yStart = 0
    ROI_coord_yEnd = 0
    Smpl_coord_xStart = 0
    Smpl_coord_xEnd = 0
    if is_file:
        with open(path + '_parameter.txt','r') as read_file:
            lines = read_file.readlines()
            # IJ.showMessage(str(lines[0]))
            if len(lines) > 1:
                file_number_string = lines[len(lines) - 1].strip()
                IJ.showMessage("File Sequence number: " + file_number_string)
                try:
                    file_number = int(file_number_string)
                except:
                    file_number = 0
                if len(lines) > 4:
                    # reading existing parameters for ROI
                    ROI = lines[len(lines) - 3].strip()
                    SmplROI = lines[len(lines) - 2].strip()
                    i1 = ROI.find("ROI")
                    if i1 != -1:
                        i1 = 13
                        i2 = ROI.find("xEnd") # + 6
                        if i2 == -1:
                            i2 = ROI.find("xEND") # + 6
                        i3 = ROI.find("yStart") # + 8
                        i4 = ROI.find("yEnd") # + 6
                        if i4 == -1:
                            i4 = ROI.find("yEND") # + 6
                        i5 = SmplROI.find("Smpl") # + 6
                        ROI_coord_xStart = int(ROI[13:i2 - 1])
                        ROI_coord_xEnd = int(ROI[i2 + 6:i3 - 1])
                        ROI_coord_yStart = int(ROI[i3 + 8:i4 - 1])
                        ROI_coord_yEnd = int(ROI[i4 + 6:len(ROI)])
                        if i5 != -1:
                            i5 = 13
                            i6 = SmplROI.find("xEnd") # + 6
                            if i6 == -1:
                                i6 = SmplROI.find("xEND") # + 6
                            Smpl_coord_xStart = int(SmplROI[13:i6 - 1])
                            Smpl_coord_xEnd =  int(SmplROI[i6 + 6:len(SmplROI)])
                            
            else:
                IJ.showMessage("No existing File Sequence")
                file_number = 0
    return file_number, ROI_coord_xStart, ROI_coord_xEnd, ROI_coord_yStart ,ROI_coord_yEnd, Smpl_coord_xStart, Smpl_coord_xEnd
#--------------------
## Main Program
RegionSE = [[0 for j in range(6)] for i in range(3)] # variable for ROI
meanIntensity = []
G_gamma2 = [0.0 for i in range(3)]
window = WM.getActiveWindow()
Mode = "Auto"
ROI_coord = {"xStart":0, "xEnd":0, "yStart":0, "yEnd":0} # for ROI coordinator data
Smpl_coord = {"xStart":0, "xEnd":0} # for sampling ROI coordinator data
if not window:
    od = OpenDialog("Choose a file")
    folder = od.getDirectory()
    filename = od.getFileName()
    if filename:
        path = folder + filename
        IJ.log(path)
        IJ.log(folder)
        IJ.log(filename)
# Always Open file with Bio-format Plugin
        imps = BF.openImagePlus(folder + filename) # Open file with Bio-format Plugin
        imp0 = imps[0] #imp0: Original Image 
        imp0.show()
    else:
        Mode ="Stop"
else:
    imp0 = IJ.getImage()
    filename = imp0.getTitle()
    folder = imp0.getOriginalFileInfo().directory  # get image's original directory
    # IJ.showMessage(folder)
    path = folder + filename
window = WM.getActiveWindow()
if not window: # if file opening is canceled 
    Mode = "Stop"

if Mode == "Auto":
#  **_paremeter.txt file exist? > read file number & ROI_coord, Smpl_coord 
    file_number, ROI_coord["xStart"], ROI_coord["xEnd"], ROI_coord["yStart"] ,ROI_coord["yEnd"], Smpl_coord["xStart"], Smpl_coord["xEnd"] = read_parameter_txt(path)
#   get Bit Depth
    bitdepth = imp0.getBitDepth()
    if bitdepth == 8:
        Mode = "8-bit"
    elif bitdepth ==16:
        Mode = "16-bit"
    elif bitdepth ==32:
        Mode = "32-bit"
    else:
        Mode = "16-bit"

#  input Parameters 0
if Mode != "Stop":
    parm = getParameters_0()
    if parm is not None:
# if parm is not null object, then...
        Mode, Show, nYDivide, mixFactor, columsWidth, smoothingBlocks, smoothingX, powerSmX, powerSmXR = parm # delete GmixRatio
#    ------------
if Mode != "Stop":
    imp0.show
    impWidth = imp0.width
    impHeight = imp0.height
    imp1 = Duplicator().run(imp0) # imp1: image for setting ROI
    imp2 = Duplicator().run(imp0) # imp2: correcting  image
    i = 0
    while i < 3:
        Title = "Select Correcting Region"
        if i == 0:
            Message = "1st ROI Selection"
        elif i == 1:
            Message = "2nd ROI Selection"
        elif i == 2:
            Message = "3rd ROI Selection"
        WM.getActiveWindow().toFront()
        if ROI_coord["yStart"] + ROI_coord["yEnd"] > 0:
            yStart = ROI_coord["yStart"]
            yEnd  = ROI_coord["yEnd"]
        else:
            yStart = 0
            yEnd  = impHeight
        parm = setRoiUI(imp0, i, Title, Message, yStart, yEnd, Mode, ROI_coord, Smpl_coord)
        if parm[0] == "Stop":
            Mode = parm[0]
            break
        else:
    # if parm is not null object, then...
            Mode, RegionSE[i][0], RegionSE[i][1], RegionSE[i][2], RegionSE[i][3], smplYes, Next = parm # Delete G_gamma2[i]
            """
            RegionSE[0][0]: xStart, RegionSE[0][1]: xEnd RegionSE[0][2]: yStart, RegionSE[0][3]: yEnd
            smplYes: Input Sampling Region, Next: Input Next Region
            """
            if smplYes == "Yes":
                Title = "Select Sampling Region"
                Message = str(i + 1) + "th Sampling Region"
                WM.getActiveWindow().toFront()
                parm2 = setRoiUI(imp0, i, Title, Message, RegionSE[i][2], RegionSE[i][3], Mode, ROI_coord, Smpl_coord)

        #  input Parameters
                if parm2[0] != "Stop":
                # if parm is not null object, then...
                    Mode, RegionSE[i][4], RegionSE[i][5] = parm2
                    """
                    RegionSE[0][4]: sampling region xStart, RegionSE[0][5]: sampling region xEnd
                    """
                else:
                    Mode = parm2[0]
                    break
            i = i + 1
            if Next == "No":
                break
#  IJ.showMessage(Mode)
timenow = datetime.datetime.now()

if Mode != "Stop":
#  -- writing "filename_parameter.txt" file
    writing_file0 = open(folder + filename  + '_parameter.txt','a')
    new_file_number = int(file_number) + 1
    i = 0
    writing_file0.write("Number of blocks: " + str(nYDivide) + " MixFactor: " + str(mixFactor) + " columsWidth: " + str(columsWidth) + "\n")
    writing_file0.write("Smoothing X?: " + str(smoothingX) + " Power Left: " + str(powerSmX) + " Power Right: " + str(powerSmXR) + "\n")
    writing_file0.write("ROI  xStart: " + str(RegionSE[i][0]) + " xEnd: " +  str(RegionSE[i][1]) + " yStart: " + str(RegionSE[i][2]) + " yEnd: " + str(RegionSE[i][3]) + "\n")
    writing_file0.write("Smpl xStart: " + str(RegionSE[i][4]) + " xEnd: " +  str(RegionSE[i][5])  + "\n")
    writing_file0.write(str(new_file_number)  + "\n")
    imp1.hide()
    manipulation(imp0, imp2, Mode, Show, RegionSE, folder, filename, nYDivide, mixFactor, columsWidth, smoothingBlocks, smoothingX, powerSmX, powerSmXR, file_number) # Delete GmixRatio, G_gamma2, 
# mixFactor: Scaling factor of correction amount
# powerSmX: Power for X direction smoothing factor(Left) powerSmXR: Power (Right)
# nYDivide: n of calcurating blocks (from parameter input dialog)
# columsWidth: Colums width for caliculation of correction value
# smoothingBlocks: execute or not smoothing for Y direction
# smoothingX: execute or not smoothing for X direction
# file_number: correction file sequence number 
    imp1.close()
    writing_file0.close()
IJ.log("Finish")
#IJ.run("Quit")

